package com.chrismweb.box2dforflashgames
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.display.Sprite
	import Box2D.Dynamics.*;
	import Box2D.Collision.*;
	import Box2D.Collision.Shapes.*;
	import Box2D.Common.Math.*;
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	
	
	/**
	 * ...
	 * #Chp4.2 - Angry Birds like game level
	 * @book by Emanuele Feronato
	 * http://www.emanueleferonato.com/
	 * Get it now @ http://www.packtpub.com/box2d-for-flash-games/book
	 */
	public class Main extends Sprite 
	{
		private var world:b2World;
		private var worldScale:Number = 30;
		
		private var theBird:Sprite=new Sprite();
		private var slingX:int=100;
		private var slingY:int=250;
		private var slingR:int=75;
		
		public function Main():void 
		{
			if (stage) 
				Init();
			else 
				addEventListener(Event.ADDED_TO_STAGE, Init);
		}
		
		private function Init(e:Event = null):void 
		{
			removeEventListener(Event.ADDED_TO_STAGE, Init);
			// entry point

			var gravity:b2Vec2 = new b2Vec2(0, 5);
			var sleep:Boolean = true;
			world = new b2World(gravity, sleep); //need to make sure you removed 'var', otherwise it will only initialize it as a local variable
			
			Setup();
			
			
			
			//stuff needed to reset example
			var reset_txt:TextField = new TextField();			
			var format:TextFormat = new TextFormat();
			format.color = 0xFFFFFFFF;
			format.align = TextFormatAlign.CENTER;
			format.size = 20;
			format.font = "Arial";
			
			reset_txt.width = 100;
			reset_txt.height = 30;
			reset_txt.selectable = false;
			reset_txt.text = "Reset";
			reset_txt.textColor = 0xFFFFFFFF;
			reset_txt.backgroundColor = 0xFF0D3451;
			reset_txt.opaqueBackground = reset_txt.backgroundColor;
			reset_txt.border = true;
			reset_txt.borderColor = 0xFFFFFFFF;
			reset_txt.defaultTextFormat = format;
			reset_txt.setTextFormat(format);
			addChild(reset_txt);
			reset_txt.addEventListener(MouseEvent.CLICK, function(e:MouseEvent):void { ResetWorld() } );
			
			DebugDraw();
			
			addEventListener(Event.ENTER_FRAME,UpdateWorld);
		}
		
		private function ResetWorld():void
		{
			trace("Resetting world");
			for (var body:b2Body=world.GetBodyList();body!=null; body=body.GetNext())
				world.DestroyBody(body);
				
			if (theBird != null)
			{
				try
				{
					theBird.removeEventListener(MouseEvent.MOUSE_DOWN, BirdClick);
					removeChild(theBird);
				}
				catch (e:ArgumentError)
				{}				
			}
				
			Setup();
		}
		private function Setup():void
		{
			Floor();
			Brick(402,431,140,36);
			Brick(544,431,140,36);
			Brick(342,396,16,32);
			Brick(604,396,16,32);
			Brick(416,347,16,130);
			Brick(532,347,16,130);
			Brick(474,273,132,16);
			Brick(474, 257, 32, 16);
			Brick(445,199,16,130);
			Brick(503,199,16,130);
			Brick(474,125,58,16);
			Brick(474,100,32,32);
			Brick(474,67,16,32);
			Brick(474,404,64,16);
			Brick(450,363,16,64);
			Brick(498,363,16,64);
			Brick(474, 322, 64, 16);
			
			
			var slingCanvas:Sprite=new Sprite();
			slingCanvas.graphics.lineStyle(1,0xffffff);
			slingCanvas.graphics.drawCircle(0,0,slingR);
			addChild(slingCanvas);
			slingCanvas.x=slingX;
			slingCanvas.y=slingY;
			theBird.graphics.lineStyle(1,0xfffffff);
			theBird.graphics.beginFill(0xffffff);
			theBird.graphics.drawCircle(0,0,15);
			addChild(theBird);
			theBird.x=slingX;
			theBird.y=slingY;
			theBird.addEventListener(MouseEvent.MOUSE_DOWN, BirdClick);
		}
		
		
		private function DebugDraw():void 
		{
			//for enabling debug draw
			var debugSprite:Sprite = new Sprite();
			addChild(debugSprite);
			
			var debugDraw:b2DebugDraw = new b2DebugDraw();
			debugDraw.SetSprite(debugSprite);
			debugDraw.SetDrawScale(worldScale);
			debugDraw.SetFlags(b2DebugDraw.e_shapeBit);
			debugDraw.SetFillAlpha(0.5);
			world.SetDebugDraw(debugDraw);
		}
		
		
		private function UpdateWorld(e:Event):void 
		{
			var timeStep:Number = 1 / 30;
			var velIterations:int=10;
			var posIterations:int=10;
			world.Step(timeStep, velIterations, posIterations);
			world.ClearForces();
			world.DrawDebugData();
		}
		private function Brick(pX:int, pY:int, w:Number, h:Number):void 
		{
			
			var bodyDef:b2BodyDef=new b2BodyDef();
			bodyDef.position.Set(PixToMeter(pX),PixToMeter(pY));
			 bodyDef.type=b2Body.b2_dynamicBody; //ctrl+q to comment/uncomment quickly in FlashDevelop
			var polygonShape:b2PolygonShape=new b2PolygonShape();
			polygonShape.SetAsBox(PixToMeter(w/2),PixToMeter(h/2));
			var fixtureDef:b2FixtureDef=new b2FixtureDef();
			fixtureDef.shape=polygonShape;
			fixtureDef.density=2;
			fixtureDef.restitution=0.4;
			fixtureDef.friction=0.5;
			var thebrick:b2Body=world.CreateBody(bodyDef);
			thebrick.CreateFixture(fixtureDef);
		}
		
		
		
		private function Floor():void 
		{
			var bodyDef:b2BodyDef=new b2BodyDef();
			bodyDef.position.Set(PixToMeter(320),PixToMeter(465));
			var polygonShape:b2PolygonShape=new b2PolygonShape();
			polygonShape.SetAsBox(PixToMeter(320),PixToMeter(15));
			var fixtureDef:b2FixtureDef=new b2FixtureDef();
			fixtureDef.shape=polygonShape;
			fixtureDef.restitution=0.4;
			fixtureDef.friction=0.5;
			var theFloor:b2Body=world.CreateBody(bodyDef);
			theFloor.CreateFixture(fixtureDef);
		}
		
		private function BirdClick(e:MouseEvent):void 
		{
			stage.addEventListener(MouseEvent.MOUSE_MOVE,BirdMove);
			stage.addEventListener(MouseEvent.MOUSE_UP,BirdRelease);
			theBird.removeEventListener(MouseEvent.MOUSE_DOWN,BirdClick);
		}
		
		private function BirdMove(e:MouseEvent):void 
		{
			theBird.x=mouseX;
			theBird.y=mouseY;
			var distanceX:Number=theBird.x-slingX;
			var distanceY:Number=theBird.y-slingY;
			if (distanceX * distanceX + distanceY * distanceY > slingR * slingR) 
			{
				var birdAngle:Number=Math.atan2(distanceY,distanceX);
				theBird.x=slingX+slingR*Math.cos(birdAngle);
				theBird.y=slingY+slingR*Math.sin(birdAngle);
			}
		}
		private function BirdRelease(e:MouseEvent):void 
		{
			var distanceX:Number=theBird.x-slingX;
			var distanceY:Number=theBird.y-slingY;
			var velocityX:Number=distanceX*-1/5;
			var velocityY:Number=distanceY*-1/5;
			var birdVelocity:b2Vec2=new b2Vec2(velocityX,velocityY);
			
			var sphereX:Number=PixToMeter(theBird.x);
			var sphereY:Number=PixToMeter(theBird.y);
			var r:Number = PixToMeter(15) ;
			
			var bodyDef:b2BodyDef=new b2BodyDef();
			bodyDef.position.Set(sphereX, sphereY);
			bodyDef.type=b2Body.b2_dynamicBody;
			
			var circleShape:b2CircleShape=new b2CircleShape(r);
			var fixtureDef:b2FixtureDef=new b2FixtureDef();
			fixtureDef.shape=circleShape;
			fixtureDef.density=4;
			fixtureDef.restitution=0.4;
			fixtureDef.friction = 0.5;
			
			var physicsBird:b2Body=world.CreateBody(bodyDef);
			physicsBird.CreateFixture(fixtureDef);
			physicsBird.SetLinearVelocity(birdVelocity);
			removeChild(theBird);
			
			trace("bird released at "+theBird.x+","+theBird.y);
			stage.removeEventListener(MouseEvent.MOUSE_MOVE,BirdMove);
			stage.removeEventListener(MouseEvent.MOUSE_UP,BirdRelease);
		}
		
		
		private function PixToMeter(pixels:Number):Number
		{
			return pixels / worldScale;
		}
	}
	
}